﻿using System.Linq;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.ChangeTracking;
using Microsoft.EntityFrameworkCore.Storage.ValueConversion;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Logging.Console;

namespace Publications {
    public class PublicationsContext : DbContext {
        public PublicationsContext (DbContextOptions<PublicationsContext> options) : base (options) { }

        public PublicationsContext () { }
        public DbSet<Magazine> Magazines { get; set; }
        public DbSet<Article> Articles { get; set; }
        public DbSet<Author> Authors { get; set; }

        public static readonly LoggerFactory MyConsoleLoggerFactory
            = new LoggerFactory (new [] {
                new ConsoleLoggerProvider ((category, level) => category == DbLoggerCategory.Database.Command.Name &&
                    level == LogLevel.Information, true)
            });

        protected override void OnConfiguring (DbContextOptionsBuilder optionsBuilder) {
            if (!optionsBuilder.IsConfigured) {
                optionsBuilder.UseSqlite (@"Filename=Data/PubsTracker.db");
                //optionsBuilder.UseSqlServer("Server=localhost;Database=Pubs;User Id=sa;Password=Passw0rd;");
            }
            optionsBuilder
                .UseLoggerFactory (MyConsoleLoggerFactory);

        }

        protected override void OnModelCreating (ModelBuilder modelBuilder) {

               modelBuilder.Entity<Magazine> ()
                .HasData (new Magazine { MagazineId = 1, Name = "MSDN Magazine" });
            modelBuilder.Entity<Magazine> ()
                .OwnsOne (m => m.Publisher)
                .HasData (new { Name = "1105 Media", YearFounded = 2006, MagazineId = 1 });

///seeding Magazine with anonymous type when 
///only ctor available doesn't take in key property
///also useful when keys (and other props) have private setters
// modelBuilder.Entity<Magazine>()
// .HasData(new {MagazineId=1, Name="MSDN Mag", Publisher="1105 Media"});


            //    modelBuilder.Entity<Magazine>()
            //    .HasData(new Magazine{MagazineId = 2,Name="The New Yorker"},
            //             new Magazine{MagazineId = 3,Name="Scientific American"}
            //    );
            
          ///Used during Guid experiment
          ///overwrite guid with a static Guid
          //var guid=new System.Guid("0483b59c-f7f8-4b21-b1df-5149fb57984e");
          //var mag1=new {MagazineId=guid, Name="MSDN Mag", Publisher="1105 Media"};
          //modelBuilder.Entity<Magazine>().HasData(mag1);
         
           ///another experiment
           // modelBuilder.Entity<Magazine>().Property(m=>m.Publisher).IsRequired();

            modelBuilder.Entity<Article> ()
                .HasData (new Article { ArticleId = 1, MagazineId = 1, Title = "EF Core 2.1 Query Types" });
            //testing FK constraint checking 
            // modelBuilder.Entity<Article> ()
            //     .HasData (new Article { AuthorId = 1, ArticleId = 2, MagazineId = 1, Title = "Creating Azure Functions That Can Read from Cosmos DB with Almost No Code" });

        }
    }
}